/*
 * Copyright (c) 2001 Will Day <willday@hpgx.net>
 *
 *    This file is part of Metamod.
 *
 *    Metamod is free software; you can redistribute it and/or modify it
 *    under the terms of the GNU General Public License as published by the
 *    Free Software Foundation; either version 2 of the License, or (at
 *    your option) any later version.
 *
 *    Metamod is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with Metamod; if not, write to the Free Software Foundation,
 *    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    In addition, as a special exception, the author gives permission to
 *    link the code of this program with the Half-Life Game Engine ("HL
 *    Engine") and Modified Game Libraries ("MODs") developed by Valve,
 *    L.L.C ("Valve").  You must obey the GNU General Public License in all
 *    respects for all of the code used other than the HL Engine and MODs
 *    from Valve.  If you modify this file, you may extend this exception
 *    to your version of the file, but you are not obligated to do so.  If
 *    you do not wish to do so, delete this exception statement from your
 *    version.
 *
 */

// the is the main API code
// please dont remove the connection message.

#include <extdll.h>
#include <h_export.h>
#include <dllapi.h>
#include <meta_api.h>
#include <cbase.h>
#include "version.h"

enginefuncs_t g_engfuncs;
globalvars_t  *gpGlobals;

extern void MyGameDLLInit( void );
extern int MyDispatchSpawn( edict_t *pEntity );

void GameDLLInit( void ) {
  MyGameDLLInit();
  SET_META_RESULT(MRES_HANDLED);
}

BOOL ClientConnect( edict_t *pEntity, const char *pszName, const char *pszAddress, char szRejectReason[ 128 ] ) {
  CLIENT_PRINTF( pEntity, print_console, "This server uses ChickenMod, visit joe.to/chicken.html for more info.\n");
  RETURN_META_VALUE(MRES_IGNORED, 0);
}

void PlayerPreThink( edict_t *pEntity ) {
  if(!FStrEq(STRING(pEntity->v.netname),"") && (FStrEq(STRING(pEntity->v.classname),"player"))) {
		if(pEntity->v.euser2 && pEntity->v.euser2 != 0) {

			pEntity->v.euser2->v.origin.x = pEntity->v.origin.x;
			pEntity->v.euser2->v.origin.y = pEntity->v.origin.y;
			pEntity->v.euser2->v.origin.z = pEntity->v.origin.z - 36;
			pEntity->v.euser2->v.angles.x = 0;
			pEntity->v.euser2->v.angles.y = pEntity->v.v_angle.y - 90;
			pEntity->v.euser2->v.angles.z = 0;

			pEntity->v.renderamt = 0;
			pEntity->v.rendermode = kRenderTransAdd;

			if(pEntity->v.deadflag == DEAD_DEAD) {
				pEntity->v.euser2->v.rendermode = kRenderTransAdd;
				pEntity->v.euser2->v.renderamt = 0;
			} else {
				pEntity->v.euser2->v.rendermode = kRenderNormal;
				pEntity->v.euser2->v.renderamt = 255;
			}

			if (pEntity->v.button & ( IN_ATTACK | IN_ATTACK2 )) {
				pEntity->v.button = 0;
			}

			if ((pEntity->v.flags & FL_DUCKING) == FL_DUCKING) {
				pEntity->v.euser2->v.origin.z = pEntity->v.origin.z - 18;
			}
		}
	}
  RETURN_META(MRES_IGNORED);
}

void PlayerPostThink( edict_t *pEntity ) {
  if(!FStrEq(STRING(pEntity->v.netname),"") && (FStrEq(STRING(pEntity->v.classname),"player"))) {
		if(pEntity->v.euser2 && pEntity->v.euser2 != 0) {
			if (pEntity->v.button & ( IN_ATTACK | IN_ATTACK2 )) {
				pEntity->v.button = 0;
			}
		}
	}
  RETURN_META(MRES_IGNORED);
}

int DispatchSpawn( edict_t *pEntity ) {
  if (FStrEq(STRING(pEntity->v.classname), "worldspawn")) {
    PRECACHE_MODEL("models/chick.mdl");
		PRECACHE_SOUND("misc/chicken0.wav");
		PRECACHE_SOUND("misc/chicken1.wav");
		PRECACHE_SOUND("misc/chicken2.wav");
		PRECACHE_SOUND("misc/chicken3.wav");
		PRECACHE_SOUND("misc/chicken4.wav");
  }
  RETURN_META_VALUE(MRES_IGNORED, 0);
}

void ClientDisconnect( edict_t *pEntity ) {
	if(pEntity->v.euser2 && pEntity->v.euser2 != 0) {
		pEntity->v.euser2->v.flags |= FL_KILLME;
		pEntity->v.euser2 = 0;
	}
  RETURN_META(MRES_IGNORED);
}

/*****************/
/* metamod stuff */
/*****************/
void WINAPI GiveFnptrsToDll( enginefuncs_t* pengfuncsFromEngine, globalvars_t *pGlobals ) {
	memcpy(&g_engfuncs, pengfuncsFromEngine, sizeof(enginefuncs_t));
	gpGlobals = pGlobals;
}

static DLL_FUNCTIONS gFunctionTable = 
{
	GameDLLInit,	// pfnGameInit
	DispatchSpawn,// pfnSpawn
	NULL,					// pfnThink
	NULL,					// pfnUse
	NULL,					// pfnTouch
	NULL,					// pfnBlocked
	NULL,					// pfnKeyValue
	NULL,					// pfnSave
	NULL,					// pfnRestore
	NULL,					// pfnSetAbsBox

	NULL,					// pfnSaveWriteFields
	NULL,					// pfnSaveReadFields

	NULL,					// pfnSaveGlobalState
	NULL,					// pfnRestoreGlobalState
	NULL,					// pfnResetGlobalState

	ClientConnect,		// pfnClientConnect
	ClientDisconnect,	// pfnClientDisconnect
	NULL,					// pfnClientKill
	NULL,					// pfnClientPutInServer
	NULL,					// pfnClientCommand
	NULL,					// pfnClientUserInfoChanged
	NULL,					// pfnServerActivate
	NULL,					// pfnServerDeactivate

	PlayerPreThink,// pfnPlayerPreThink
	PlayerPostThink,// pfnPlayerPostThink

	NULL,					// pfnStartFrame
	NULL,					// pfnParmsNewLevel
	NULL,					// pfnParmsChangeLevel

	NULL,					// pfnGetGameDescription
	NULL,					// pfnPlayerCustomization

	NULL,					// pfnSpectatorConnect
	NULL,					// pfnSpectatorDisconnect
	NULL,					// pfnSpectatorThink
	
	NULL,					// pfnSys_Error

	NULL,					// pfnPM_Move
	NULL,					// pfnPM_Init
	NULL,					// pfnPM_FindTextureType
	
	NULL,					// pfnSetupVisibility
	NULL,					// pfnUpdateClientData
	NULL,					// pfnAddToFullPack
	NULL,					// pfnCreateBaseline
	NULL,					// pfnRegisterEncoders
	NULL,					// pfnGetWeaponData
	NULL,					// pfnCmdStart
	NULL,					// pfnCmdEnd
	NULL,					// pfnConnectionlessPacket
	NULL,					// pfnGetHullBounds
	NULL,					// pfnCreateInstancedBaselines
	NULL,					// pfnInconsistentFile
	NULL,					// pfnAllowLagCompensation
};

C_DLLEXPORT int GetEntityAPI2( DLL_FUNCTIONS *pFunctionTable, int *interfaceVersion ) {
	if(!pFunctionTable) {
		return(FALSE);
	}
	else if(*interfaceVersion != INTERFACE_VERSION) {
		*interfaceVersion = INTERFACE_VERSION;
		return(FALSE);
	}
	memcpy( pFunctionTable, &gFunctionTable, sizeof( DLL_FUNCTIONS ) );
	return(TRUE);
}

plugin_info_t Plugin_info = {
	META_INTERFACE_VERSION,
	MY_NAME,
	MY_VERSION,
	MY_DATE,
	MY_AUTHOR,
	MY_URL,
	MY_LOGTAG,
	PT_STARTUP,
	PT_STARTUP,
};

static META_FUNCTIONS gMetaFunctionTable = {
	NULL,
	NULL,
	GetEntityAPI2,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
};

meta_globals_t *gpMetaGlobals;
gamedll_funcs_t *gpGamedllFuncs;
mutil_funcs_t *gpMetaUtilFuncs;

C_DLLEXPORT int Meta_Query(char *ifvers, plugin_info_t **pPlugInfo, mutil_funcs_t *pMetaUtilFuncs) {
	*pPlugInfo=&Plugin_info;
	gpMetaUtilFuncs=pMetaUtilFuncs;
	return(TRUE);
}

C_DLLEXPORT int Meta_Attach(PLUG_LOADTIME now, META_FUNCTIONS *pFunctionTable, meta_globals_t *pMGlobals, gamedll_funcs_t *pGamedllFuncs) {
	if(!pMGlobals) {
		LOG_ERROR(PLID, "Meta_Attach called with null pMGlobals");
		return(FALSE);
	}
	gpMetaGlobals=pMGlobals;
	if(!pFunctionTable) {
		LOG_ERROR(PLID, "Meta_Attach called with null pFunctionTable");
		return(FALSE);
	}
	memcpy(pFunctionTable, &gMetaFunctionTable, sizeof(META_FUNCTIONS));
	gpGamedllFuncs=pGamedllFuncs;
	return(TRUE);
}

C_DLLEXPORT int Meta_Detach(PLUG_LOADTIME now, PL_UNLOAD_REASON reason) {
	if(now && reason) {
	  return(TRUE);
	} else {
	  return(FALSE);
	}
}
